#ifndef _INTELHEX_H_
#define _INTELHEX_H_

#include <stdint.h>
#include <stddef.h>
#include <string.h>


/* - Constants ------------------------------------------------------------ */

// Buffer lengths for data and record lines
#define IHEX_DATAMAX                255     // One line's worth of input data
#define IHEX_LINEMIN                12      // Min length of a record line
#define IHEX_LINEMAX                522     // Max length of a record line

// Record line format:
// ':'<Len><Addr><Type><Data><CkSum><NULL> == 1 + 2 + 4 + 2 + (<255)*2 + 2 + 1

// Record Types
#define IHEX_RTYPE_DATA             0x00    // Data record
#define IHEX_RTYPE_EOF              0x01    // End of file
#define IHEX_RTYPE_EXTADDR16        0x02    // Extended Segment Address (16b)
#define IHEX_RTYPE_STARTADDR16      0x03    // Start Segment Address (not used)
#define IHEX_RTYPE_EXTADDR32        0x04    // Extended Linear Address (32b)
#define IHEX_RTYPE_STARTADDR32      0x05    // Start Linear Address (not used)
#define IHEX_RTYPE_INVALID          0xFF    // Marker for an invalid line


/* - Function Prototypes -------------------------------------------------- */

uint8_t  ihex_get_record_type   (const char *in);
void     ihex_encode_raw        (const char *data, char *out, uint8_t len, uint16_t addr, uint8_t rtype);
void     ihex_encode_eof        (char *out);
void     ihex_encode_ext_addr   (char *out, uint8_t rtype, uint16_t addr);
uint16_t ihex_decode_ext_addr   (const char *in);
void     ihex_encode_start_addr (char *out, uint8_t rtype, uint32_t addr);
uint32_t ihex_decode_start_addr (const char *in);
void     ihex_encode_data       (const char *in, char *out, uint8_t len, uint16_t addr);
uint8_t  ihex_decode_data       (const char *in, char *out, uint16_t *addr);
uint8_t  ihex_checksum          (const char *in, uint8_t len);
void     ihex_hex               (char *buf, uint8_t val);
uint8_t  ihex_int               (const char *buf);

#endif // _INTELHEX_H_