#ifndef _unix_h_
#define _unix_h_

#include <stdint.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <inttypes.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>


/* - Constants ------------------------------------------------------------ */

// Version info
#define VMAJOR              0
#define VMINOR              1
#define VREV                11

// Buffer length for line reads
#define BUFLEN              80

// Flag constants
#define FLAG_HELP           0x01    // Show help and exit
#define FLAG_VERSION        0x02    // Show version and exit
#define FLAG_FORCE          0x10    // Skip ID check
#define FLAG_ERASE          0x20    // Erase flash before writing

// Mode constants
#define MODE_QUERY          0x01    // Run part query and exit
#define MODE_CHECK_PID      0x02    // Query/verify part ID
#define MODE_CHECK_UID      0x04    // Query/verify unique ID
#define MODE_CHECK_ROM      0x08    // Query/verify ROM version
#define MODE_FLASH          0x10    // Run flash operations
#define MODE_FLASH_WRITE    0x20    // Run flash write operations


/* - Type Declarations ---------------------------------------------------- */

struct cfgopt_t {
    char     *port;         // Pointer to port device string in argv[]
    uint32_t  baud;         // Baudrate
    uint8_t   stop;         // Stop bits
    uint8_t   attempts;     // Sync attempts
    uint32_t  clock;        // Clock frequency (kHz)
    uint32_t  pid;          // Part ID
    uint32_t  uid[4];       // Unique ID
    uint8_t   romver[2];    // ROM version
    uint16_t  lock;         // Unlock code
};

struct baudpair_t {
    uint32_t  baud;         // Baud rate as value
    int       constant;     // Baud rate as constant
};


/* - Function Prototypes -------------------------------------------------- */

// From unix.c
int  main            (int argc, char *argv[]);
void show_usage      (char *prog);
void show_help       ();
void show_version    ();
void spinner         (uint8_t spin);
int  get_baud        (uint32_t *baud);
void set_port_config (int fd, int baudconst, uint8_t stop);

// From unix-cmds.c
int  waitfor         (int fd, char *data);
int  readline        (int fd, char *buf, unsigned buflen);
int  synchronize     (int fd);
int  set_clock       (int fd, uint32_t clock);
int  echo_off        (int fd);
int  set_new_speed   (int fd, uint32_t baud, uint8_t stop);
int  get_part_id     (int fd, uint32_t *id);
int  get_unique_id   (int fd, uint32_t *id);
int  get_rom_version (int fd, uint8_t *ver);
int  unlock          (int fd, uint32_t code);
int  check_blank     (int fd, uint8_t start, uint8_t len);

#endif // _unix_h_